﻿' VB.NET to Add Arc Text
'   lines starting with an apostrophe are comments

'   Used for calculations
Imports System.Math

Public Class ArcText

    ' create Visual CADD object with events
    Shared WithEvents vcadd As New VCadd32.Automation

    ' create point objects and variables
    Shared ptCenter As New VCadd32.VCPoint2D
    Shared ptRadius As New VCadd32.VCPoint2D
    Shared pt0 As New VCadd32.VCPoint2D
    Shared pt1 As New VCadd32.VCPoint2D
    Shared dRadius As Double
    Shared dAngle As Double
    Shared hEntRubber As Long = -1

    Public Shared Function Run(ByVal str As String) As Integer
        '   ArcText tool will draw text curved along an arc entered 
        '   by the user.  The tool has 3 states:
        '       (1) place center point of the arc
        '       (2) place radius and text start point
        '       (3) type text
        '   ArcText supports the BackSpace key and undo/redo.
        '   ArcText is placed as a group of individual characters.

        '   Optional - save the settings to be restored later
        vcadd.SaveSettings()

        '   Increment the undo level to support undo/redo.
        '   Optional - use VCBeginOperation() instead.  
        '       VCBeginOperation() does only two things - it 
        '       increments the undo level and saves the current end 
        '       of the database for cleanup when done.  The ArcText 
        '       tool uses a rubberband which is appended at the end
        '       of the database, so we can use that entity handle for 
        '       cleanup instead.
        vcadd.IncrementUndoLevel()

        '   Start a user tool and define the status bar command line
        '   prompts for the ArcText tool.  The states are 0, 1, 2.
        '   The first prompt for state 0 is set by VCSetUserTool()
        '   and the next two prompts are set by VCSetPrompt().
        vcadd.SetUserTool(2, "ArcText", "Enter Center Point")
        vcadd.SetPrompt(1, "Enter Start Point of Text")
        vcadd.SetPrompt(2, "Enter Text, press [Back] key to re-type, [Enter] key to finish, [Esc] key to abort")

        '   Set alert events so our tool gets notifications from 
        '   Visual CADD about user interface events, other tool
        '   operations, and Windows messages.  We are requesting 
        '   alerts on mouse-down (point placement), mouse-move (for
        '   rubberbands), pen up (when tool ends normally),
        '   abort (when user aborts or another tool starts), and
        '   Windows messages.  Each of these has a corresponding 
        '   callback function below.
        vcadd.SetAlertEvent(ALERT_APP_UTOOL_MOUSEDOWN + ALERT_APP_UTOOL_MOUSEMOVE + ALERT_APP_UTOOL_PENUP + ALERT_APP_UTOOL_ABORT, ALERT_APP_EX_PRE_TRANSLATE_GET_MESSAGE)

        '   Attach the alert events to the handler functions
        AddHandler vcadd.MouseDown, AddressOf vcadd_MouseDown
        AddHandler vcadd.MouseMove, AddressOf vcadd_MouseMove
        AddHandler vcadd.PenUp, AddressOf vcadd_PenUp
        AddHandler vcadd.Abort, AddressOf vcadd_Abort
        AddHandler vcadd.PreTranslateGetMessage, AddressOf vcadd_PreTranslateGetMessage

        ' return
        Return True

    End Function

    Private Shared Function vcadd_PreTranslateGetMessage(ByRef hWnd As Integer, ByRef uMsg As Integer,
                                                    ByRef wParam As Integer, ByRef lParam As Integer,
                                                    ByRef Time As Integer, ByRef PointX As Integer,
                                                    ByRef PointY As Integer) As Integer
        '   PreTranslateGetMessage processes posted messages, usually
        '   keyboard messages, but any posted messages may be 
        '   processed.

        '   The function return will tell Visual CADD if we have 
        '   processed the message and whether it should process it.
        '   The default is FALSE, we have not processed it.  Setting 
        '   to TRUE tells Visual CADD to not process the message.

        '   get the tool state so we know the state of the tool
        Dim iState As Short
        iState = vcadd.GetUserToolState()

        '   &H100 is the WM_KEYDOWN and 27 is VK_ESCAPE or Esc key.
        '   Abort the ArcText tool anytime the user presses Esc.
        If ((uMsg = &H100) And (wParam = 27)) Then
            vcadd.Abort()
            Return True
        End If

        '   Once the tool reaches state 2, we will process keyboard
        '   input as the user types.
        If (iState = 2) Then
            If (uMsg = &H100) Then
                '   &H100 is WM_KEYDOWN
                If (wParam = 13) Then
                    '   13 is VK_RETURN or Enter key.
                    '   Calling VCPenUp() ends a tool normally.
                    vcadd.PenUp()
                    Return True
                ElseIf (wParam = 8) Then
                    '   8 is VK_BACK or backspace key.
                    '   Our BackSpace function will remove the last
                    '   typed character.
                    BackSpace()
                    Return True
                End If
            ElseIf (uMsg = &H102) Then
                '   &H102 is WM_CHAR
                If (wParam >= 32) Then
                    '   characters 32 and above are printable 
                    '   so we add them to our ArcText by calling our 
                    '   TextIt function.
                    TextIt(Chr(wParam))
                End If
                Return True
            End If
        End If

        '   We have not processed the message, let Visual CADD.
        Return False

    End Function

    Private Shared Sub vcadd_MouseDown(wParam As Integer, lParam As Integer)
        '   get the tool state so we know the state of the tool
        Dim iState As Short
        iState = vcadd.GetUserToolState()

        '   Visual CADD increases the tool state of user tools after 
        '   mouse clicks.  Tool state 1 means the tool state was 
        '   previously 0 and the user has clicked for the center of
        '   the arc.     
        If (iState = 1) Then
            '   get the mouse-click point from Visual CADD
            ptCenter = vcadd.GetUserToolLBDown()

            '   add circle and line (radius) entities to be the 
            '   rubberband.  Save the rubberband entity handle for
            '   later use.
            vcadd.AddCircleEntity(-1, ptCenter, ptCenter)
            hEntRubber = vcadd.LastEntity()
            vcadd.AddLineEntity(-1, ptCenter, ptCenter)
        End If

        '   Tool state 2 means the tool state was previously 1 and 
        '   the user has clicked for the start of the text.
        If (iState = 2) Then
            '   get the mouse-click point from Visual CADD
            ptRadius = vcadd.GetUserToolLBDown()

            '   calculate the radius distance and the angle.
            Dim dX As Double
            Dim dY As Double

            dX = ptRadius.X - ptCenter.X
            dY = ptRadius.Y - ptCenter.Y

            dRadius = Sqrt(dX * dX + dY * dY)
            dAngle = Atan2(dY, dX)
        End If

        '   Tool state 3 means the tool state was previously 2 and 
        '   the user has clicked to finish the tool.
        If (iState = 3) Then
            vcadd.PenUp()
        End If

    End Sub

    Private Shared Sub vcadd_MouseMove(wParam As Integer, lParam As Integer)
        '   get the tool state so we know the state of the tool
        Dim iState As Short
        iState = vcadd.GetUserToolState()

        '   use the mouse move to draw the rubberband while the user
        '   is choosing the arc start point.  Tool state 1 means the 
        '   user has already clicked the center so we will draw the
        '   rubberband to the radius at the current mouse location.
        If (iState = 1) Then
            '   drawing rubberbands involves erasing the old 
            '   rubberband and drawing the new one.  The function
            '   VCDrawCurrentEntityXOR() is used to draw entities 
            '   using the user's rubberband color.  The XOR means 
            '   if it is already drawn on-screen XOR will remove it; 
            '   if it is not already drawn on-screen XOR will draw it.

            '   erase the circle and line (radius) rubberbands
            vcadd.SetCurrentEntity(hEntRubber)
            vcadd.DrawCurrentEntityXOR()

            vcadd.SetCurrentEntity(hEntRubber + 1)
            vcadd.DrawCurrentEntityXOR()

            '   get the mouse location point from Visual CADD
            ptRadius = vcadd.GetUserToolMouseMove()

            '   set the new point in the circle and line (radius) 
            '   rubberband entities and draw them
            vcadd.SetCurrentEntity(hEntRubber)
            vcadd.SetCurrentEntityPoint(1, ptRadius)
            vcadd.DrawCurrentEntityXOR()

            vcadd.SetCurrentEntity(hEntRubber + 1)
            vcadd.SetCurrentEntityPoint(1, ptRadius)
            vcadd.DrawCurrentEntityXOR()
        End If

    End Sub

    Private Shared Sub TextIt(ByVal strText As String)
        '   TextIt is called by the PreTranslateGetMessage after we 
        '   are in state 2, meaning we are accepting text input.

        '   create variables for calculation 
        Dim strChar As String
        Dim hEntLast As Long
        Dim iLen As Short
        Dim i As Short
        Dim dSpan As Double
        Dim dWidth As Double

        '   We are using the TextIt function to process one character
        '   at a time.  However, the function can also process full 
        '   length text strings.  We get the length of the string, 
        '   1 in the case of the current ArcText tool.
        iLen = strText.Length

        '   Loop through the string one character at a time
        For i = 0 To iLen - 1
            '   get a single character and set Visual CADD text to 
            '   use that character
            strChar = strText.Chars(i)
            vcadd.SetTextString(strChar)

            '   Total character width is the width of the text entity
            '   using that character plus character spacing
            dWidth = GetStringWidth()

            '   Calculate the text placement point from the center,
            '   radius, and angle
            pt0.X = ptCenter.X + dRadius * Cos(dAngle)
            pt0.Y = ptCenter.Y + dRadius * Sin(dAngle)

            '   Calculate how much angle is spanned by the character
            '   width
            dSpan = Atan2(dWidth, dRadius)

            '   The standard angle in Visual CADD is measured 
            '   counter-clockwise from 3 o'clock whereas text 
            '   rotation angle of 0.0 means upright text, or 12 
            '   o'clock, a difference of 90 degrees.  The API uses 
            '   radians for angles, so 90 degrees is pi / 2 and we 
            '   adjust the text rotation angle by that amount.
            vcadd.SetTextRot(dAngle - dSpan / 2.0 - HALFPI)

            '   spacing requires left-justified text
            vcadd.SetTextJustify(Asc("L"))

            '   Add the text at the calculated point
            vcadd.AddTextEntity(-1, pt0)

            '   Adjust the angle clockwise for the next character
            dAngle = dAngle - dSpan

            '   Draw the newly placed character
            hEntLast = vcadd.LastEntity()
            vcadd.SetCurrentEntity(hEntLast)
            vcadd.DrawCurrentEntity()
        Next

    End Sub

    Private Shared Function GetCharacterSpace() As Double
        '   Get the character space setting so we can calculate
        '   how to space the text around the arc.  

        Dim dCharSp As Double

        If (vcadd.GetTextFontType() = 1) Then
            '   Font type 1 is a vector font.  The character spacing
            '   is a percentage of the text height.
            Dim dHeight As Double
            Dim dCharSpPerCent As Double
            dHeight = vcadd.GetTextHeight()
            dCharSpPerCent = vcadd.GetTextCharSpace()
            dCharSp = dHeight * dCharSpPerCent
        Else
            '   Font type 0 is a TrueType font.  The character 
            '   spacing is built directly into TrueType character 
            '   definitions so there is no additional spacing.
            dCharSp = 0.0
        End If

        Return dCharSp

    End Function

    Private Shared Function GetStringWidth() As Double
        '   use rotation angle 0.0 to place a temporary text 
        '   entity we can use to find the width of the character.
        vcadd.SetTextRot(0.0)
        vcadd.AddTextEntity(-1, ptCenter)

        '   VCEntityExtents() gets the size of an entity.  After
        '   we get the size, we can use VCTruncFrom() to delete
        '   this temporary entity.
        Dim hEntLast As Long
        hEntLast = vcadd.LastEntity()
        vcadd.EntityExtents(hEntLast, pt0, pt1)
        vcadd.TruncFrom(hEntLast)

        '   Total character width is the width of the text entity
        '   using that character plus character spacing
        Return (pt1.X - pt0.X + GetCharacterSpace())

    End Function

    Private Shared Sub BackSpace()
        '   BackSpace is called by the PreTranslateGetMessage after
        '   we are in state 2, meaning we are accepting text input,
        '   and the user has pressed the Back key.

        '   create variables for calculation 
        Dim hEntLast As Long
        Dim dSpan As Double
        Dim dWidth As Double

        '   Get the last entity handle
        hEntLast = vcadd.LastEntity()

        '   Before processing a backspace key, be sure our text 
        '   string is not already empty.  The rubberband is two
        '   entities, so the last entity must be after the second
        '   rubberband entity.
        If (hEntLast > hEntRubber + 1) Then

            '   Match the last entity so we can use its text string, 
            '   actually an individual character
            vcadd.SetCurrentEntity(hEntLast)
            vcadd.MatchCurrentEntity()

            '   Total character width is the width of the text entity
            '   using that character plus character spacing
            dWidth = GetStringWidth()

            '   Calculate how much angle is spanned by the character
            '   width and adjust the angle counter-clockwise back 
            '   to the previous character.
            dSpan = Atan2(dWidth, dRadius)
            dAngle = dAngle + dSpan

            '   Erase the last entity, draw it so it erases from the
            '   screen, and truncate the database to permanently 
            '   remove it.
            hEntLast = vcadd.LastEntity()
            vcadd.SetCurrentEntity(hEntLast)
            vcadd.SetCurrentErased()
            vcadd.DrawCurrentEntity()
            vcadd.TruncFrom(hEntLast)
        End If

    End Sub

    Private Shared Sub vcadd_PenUp(wParam As Integer, lParam As Integer)
        '   ArcText tool has completed normally, so we need to 
        '   cleanup the database and do final processing.

        '   Get the last entity handle
        Dim hEntLast As Long
        hEntLast = vcadd.LastEntity()

        '   Check that the user has entered some text, not just
        '   pressed Enter.
        If ((hEntRubber = -1) Or (hEntLast < hEntRubber + 2)) Then
            '   There was no text entered, so Abort instead.
            vcadd.Abort()
        Else
            '   Call the CleanUp function to handle routine 
            '   processing
            CleanUp()

            '   clear selection
            vcadd.ClearSelection()

            '   Loop through the text entities and select them
            For hEnt = hEntRubber + 2 To hEntLast
                vcadd.SetCurrentEntity(hEnt)
                vcadd.SetCurrentSelected()
            Next

            '   Purge the database to cleanup redo conflicts possibly
            '   left by previous tools.  Optional - if you used 
            '   VCBeginOperation(), then you should call 
            '   VCEndOperation() which does the same VCPurgeRedo().
            vcadd.PurgeRedo()

            '   Group the ArcText entities so they stay together
            '   when selected
            vcadd.GroupSelected()
        End If

    End Sub

    Private Shared Sub vcadd_Abort(wParam As Integer, lParam As Integer)
        '   ArcText tool has aborted, so we need to cleanup the 
        '   database.

        '   Call the CleanUp function to handle routine 
        '   processing
        CleanUp()

        '   If ArcText made it as far as a rubberband, then erase
        '   from the screen any text.
        If (hEntRubber <> -1) Then
            '   Get the last entity handle
            Dim hEntLast As Long
            hEntLast = vcadd.LastEntity()

            '   Loop through the text entities
            For hEnt = hEntRubber + 2 To hEntLast
                '   Erase each text entity and draw it on-screen
                '   to erase it from screen
                vcadd.SetCurrentEntity(hEnt)
                vcadd.SetCurrentErased()
                vcadd.DrawCurrentEntity()
            Next

            '   Truncate the database starting with the rubberband
            '   to restore the database to its original condition.
            vcadd.TruncFrom(hEntRubber)
        End If

        '   Decrement the undo level since the ArcText was aborted 
        '   and no entities were added.  Optional - if you used 
        '   VCBeginOperation(), then you should call 
        '   VCAbortOperation() which does the same VCTruncFrom() and 
        '   VCDecrementUndoLevel().
        vcadd.DecrementUndoLevel()
    End Sub

    Private Shared Sub CleanUp()
        '   If ArcText made it as far as a rubberband, then erase
        '   from the screen the two rubberband entities.  CleanUp is
        '   called by both ToolComplete and ToolAbort, so we do not
        '   know whether the database will be purged or truncated.
        '   Thus we use VCSetCurrentEntityPurged() to both erase the
        '   entities and mark them for purging.  Draw the erased
        '   entities to remove them from the screen.
        If hEntRubber <> -1 Then
            vcadd.SetCurrentEntity(hEntRubber)
            vcadd.SetCurrentEntityPurged(1)
            vcadd.DrawCurrentEntity()

            vcadd.SetCurrentEntity(hEntRubber + 1)
            vcadd.SetCurrentEntityPurged(1)
            vcadd.DrawCurrentEntity()
        End If

        '   We are done so clear the alert events so Visual CADD
        '   no longer sends the alerts.
        vcadd.ClearAlertEvent()

        '   Remove the alert events from the handler functions
        RemoveHandler vcadd.MouseDown, AddressOf vcadd_MouseDown
        RemoveHandler vcadd.MouseMove, AddressOf vcadd_MouseMove
        RemoveHandler vcadd.PenUp, AddressOf vcadd_PenUp
        RemoveHandler vcadd.Abort, AddressOf vcadd_Abort
        RemoveHandler vcadd.PreTranslateGetMessage, AddressOf vcadd_PreTranslateGetMessage

        '   Restore the saved settings so the user settings are as 
        '   they began.
        vcadd.RestoreSettings()
    End Sub
End Class
